from dataclasses import dataclass
from environs import Env


@dataclass
class TgBot:
    token: str
    admin_ids: list
    admin_logins: list
    channels_ids: list
    channels_links: list
    name_second_bot: str
    number_of_referrals: int


@dataclass
class Miscellaneous:
    other_params: str = None


@dataclass
class Config:
    tg_bot: TgBot
    misc: Miscellaneous


def load_config(path: str = None):
    env = Env()
    env.read_env(path)

    return Config(
        tg_bot=TgBot(
            token=env.str("BOT_TOKEN"),
            admin_ids=list(map(int, env.list("ADMINS"))),
            admin_logins=list(map(str, env.list("ADMIN_LOGINS"))),
            channels_ids=list(map(int, env.list("CHANNELS_IDS"))),
            channels_links=list(map(str, env.list("CHANNELS_LINKS"))),
            name_second_bot=env.str("NAME_SECOND_BOT"),
            number_of_referrals=env.int("NUMBER_OF_REFERRALS")
        ),
        misc=Miscellaneous()
    )
