from aiogram import types
from aiogram.dispatcher import FSMContext
from filters.all_filters import IsPrivate, IsAdmin
from keyboards.default.admin_keyboard import back_button, start_menu, confirm_menu, confirm_advertising_menu
from keyboards.inline.admin_inline import add_new_ads, send_mailing_keyboard, send_advertising_keyboard
from states.all_states import AllPages, MailingMenu, ChangeAdvertisement
from utils.other_functions import *


@dp.message_handler(IsPrivate(), IsAdmin(), text='💼 Меню администратора', state=AllPages.page_1)
async def open_stats(message: types.message):
    text_stat = await generate_stats()
    await dp.bot.send_message(message.from_user.id, text=text_stat, reply_markup=add_new_ads())


@dp.message_handler(IsPrivate(), IsAdmin(), text='📢 Рассылка', state=AllPages.page_1)
async def make_a_newsletter(message: types.message):
    await dp.bot.send_message(message.from_user.id,
                              text='<b>📢Рассылка</b>\n\nЧтобы сделать рассылку введите текст!',
                              reply_markup=back_button())
    await MailingMenu.get_message.set()


@dp.message_handler(IsPrivate(), IsAdmin(), text=['◀Назад', '❌Отмена'],
                    state=[MailingMenu.get_message, MailingMenu.confirm_menu_state, MailingMenu.get_photo_state,
                           MailingMenu.get_video_state])
async def return_main_menu(message: types.message):
    await dp.bot.send_message(message.from_user.id,
                              text=f'<b>❌Рассылка отменена</b>', reply_markup=start_menu())
    await AllPages.page_1.set()


@dp.message_handler(IsPrivate(), IsAdmin(), text=['◀Назад', '❌Отмена'],
                    state=[ChangeAdvertisement.get_url, ChangeAdvertisement.get_photo, ChangeAdvertisement.get_text,
                           ChangeAdvertisement.confirm_state])
async def return_main_menu_second(message: types.message):
    await dp.bot.send_message(message.from_user.id,
                              text=f'<b>❌Изменение рекламы отменено</b>', reply_markup=start_menu())
    await AllPages.page_1.set()


@dp.message_handler(IsPrivate(), IsAdmin(), state=MailingMenu.get_message)
async def get_mailing_message(message: types.message, state: FSMContext):
    await dp.bot.send_message(message.from_user.id,
                              text=f'<b>Хотите добавить фото/видео к рассылке?</b>',
                              reply_markup=send_mailing_keyboard())
    await state.update_data(message_text=message.text)
    await MailingMenu.get_material_answer.set()


@dp.callback_query_handler(IsAdmin(), text='skip_add_material', state=MailingMenu.get_material_answer)
async def open_the_confirmation_menu(callback: types.CallbackQuery, state: FSMContext):
    await callback.message.delete()
    message_text = await state.get_data('message_text')
    await state.update_data(type_send='text')
    await dp.bot.send_message(callback.from_user.id,
                              text=f'<b>Вы уверены, что хотите отправить это сообщение:</b>\n\n{message_text["message_text"]}',
                              reply_markup=confirm_menu())
    await MailingMenu.confirm_menu_state.set()


@dp.callback_query_handler(IsAdmin(), text='add_photo', state=MailingMenu.get_material_answer)
async def add_photo_mailing(callback: types.CallbackQuery):
    await callback.message.delete()
    await dp.bot.send_message(callback.from_user.id, text='<b>Отправьте фото для рассылки!</b>',
                              reply_markup=back_button())
    await MailingMenu.get_photo_state.set()


@dp.callback_query_handler(IsAdmin(), text='add_video', state=MailingMenu.get_material_answer)
async def add_video_mailing(callback: types.CallbackQuery):
    await callback.message.delete()
    await dp.bot.send_message(callback.from_user.id, text='<b>Отправьте видео для рассылки!</b>',
                              reply_markup=back_button())
    await MailingMenu.get_video_state.set()


@dp.callback_query_handler(IsAdmin(), text='edit_ads', state=AllPages.page_1)
async def edit_advertising(callback: types.CallbackQuery):
    await callback.message.delete()
    await dp.bot.send_message(callback.from_user.id,
                              text='<b>⚙Изменение рекламы</b>\n\nВведите рекламный текст!',
                              reply_markup=back_button())
    await ChangeAdvertisement.get_text.set()


@dp.message_handler(IsPrivate(), IsAdmin(), state=ChangeAdvertisement.get_text)
async def get_advertising_text(message: types.message, state: FSMContext):
    await dp.bot.send_message(message.from_user.id,
                              text=f'<b>Хотите добавить фото к рекламе?</b>',
                              reply_markup=send_advertising_keyboard())
    await state.update_data(message_text=message.text)
    await ChangeAdvertisement.add_photo_answer.set()


@dp.callback_query_handler(IsAdmin(), text='add_advertising_photo', state=ChangeAdvertisement.add_photo_answer)
async def add_photo_advertising(callback: types.CallbackQuery):
    await callback.message.delete()
    await dp.bot.send_message(callback.from_user.id, text='<b>Отправьте фото для рекламы!</b>',
                              reply_markup=back_button())
    await ChangeAdvertisement.get_photo.set()


@dp.message_handler(IsPrivate(), IsAdmin(), state=ChangeAdvertisement.get_photo, content_types=types.ContentType.PHOTO)
async def get_advertising_photo(message: types.message, state: FSMContext):
    photo_file_id = message.photo[-1].file_id
    await dp.bot.send_message(message.from_user.id, text='<b>Отлично, теперь отправьте ссылку на канал/бота!</b>',
                              reply_markup=back_button())
    await state.update_data(type_send='photo')
    await state.update_data(photo=photo_file_id)
    await ChangeAdvertisement.get_url.set()


@dp.callback_query_handler(IsAdmin(), text='skip_add_advertising_photo', state=ChangeAdvertisement.add_photo_answer)
async def skip_add_photo_advertising(callback: types.CallbackQuery, state: FSMContext):
    await callback.message.delete()
    await dp.bot.send_message(callback.from_user.id, text='<b>Отлично, теперь отправьте ссылку на канал/бота!</b>',
                              reply_markup=back_button())
    await state.update_data(type_send='text')
    await ChangeAdvertisement.get_url.set()


@dp.message_handler(IsPrivate(), IsAdmin(), state=ChangeAdvertisement.get_url)
async def get_advertising_url(message: types.message, state: FSMContext):
    url = message.text
    message_text = await state.get_data('message_text')
    photo = await state.get_data('photo')
    type_send = await state.get_data('type_send')
    if type_send['type_send'] == 'text':
        await dp.bot.send_message(message.from_user.id, text=f'<b>Рекламный пост:</b>\n\n'
                                                             f'{message_text["message_text"]}\n\n'
                                                             f'<b>Ссылка на канал/бота:</b>\n'
                                                             f'{url}',
                                  reply_markup=confirm_advertising_menu())
    elif type_send['type_send'] == 'photo':
        await dp.bot.send_photo(message.from_user.id, photo=photo["photo"],
                                caption=f'{message_text["message_text"]}\n\n'
                                        f'<b>Ссылка на канал/бота:</b>\n'
                                        f'{url}',
                                reply_markup=confirm_advertising_menu())
    await state.update_data(url=url)
    await ChangeAdvertisement.confirm_state.set()


@dp.message_handler(IsPrivate(), IsAdmin(), state=MailingMenu.get_photo_state, content_types=types.ContentType.PHOTO)
async def get_mailing_photo(message: types.message, state: FSMContext):
    photo_file_id = message.photo[-1].file_id
    message_text = await state.get_data('message_text')
    await dp.bot.send_photo(message.from_user.id,
                            photo=photo_file_id,
                            caption=f'{message_text["message_text"]}',
                            reply_markup=confirm_menu())
    await state.update_data(type_send='photo')
    await state.update_data(photo=photo_file_id)
    await MailingMenu.confirm_menu_state.set()


@dp.message_handler(IsPrivate(), IsAdmin(), state=MailingMenu.get_video_state, content_types=types.ContentType.VIDEO)
async def get_mailing_video(message: types.message, state: FSMContext):
    video_file_id = message.video.file_id
    message_text = await state.get_data('message_text')
    await dp.bot.send_video(message.from_user.id,
                            video=video_file_id,
                            caption=f'{message_text["message_text"]}',
                            reply_markup=confirm_menu())
    await state.update_data(type_send='video')
    await state.update_data(video=video_file_id)
    await MailingMenu.confirm_menu_state.set()


@dp.message_handler(IsPrivate(), IsAdmin(), text='✅Отправить', state=MailingMenu.confirm_menu_state)
async def send_a_mailing(message: types.message, state: FSMContext):
    message_text = await state.get_data('message_text')
    type_send = await state.get_data('type_send')
    photo = await state.get_data('photo')
    video = await state.get_data('video')
    count = await send_a_newsletter(message_text['message_text'], type_send['type_send'], photo, video)
    await dp.bot.send_message(message.from_user.id,
                              text=f'<b>✅Рассылка успешно завершена</b>\n'
                                   f'Получили {count} пользователей', reply_markup=start_menu())
    await AllPages.page_1.set()


@dp.message_handler(IsPrivate(), IsAdmin(), text='✅Изменить', state=ChangeAdvertisement.confirm_state)
async def edit_advertising_confirm(message: types.message, state: FSMContext):
    message_text = await state.get_data('message_text')
    url = await state.get_data('url')
    photo = await state.get_data('photo')
    type_send = await state.get_data('type_send')
    await dp.bot.send_message(message.from_user.id,
                              text=f'<b>✅Изменение рекламы успешно завершено</b>', reply_markup=start_menu())
    if type_send['type_send'] == 'text':
        update_ads(advertising_text=message_text['message_text'], url_advertising_channel=url['url'], photo_id='0')
    else:
        update_ads(advertising_text=message_text['message_text'], url_advertising_channel=url['url'],
                   photo_id=photo['photo'])
    await AllPages.page_1.set()
