from aiogram import types
from aiogram.utils.deep_linking import get_start_link
from data.config import load_config
from filters.all_filters import IsPrivate, IsSubscriber
from keyboards.default.user_keyboard import choosing_subscriptions, last_page, next_page
from keyboards.inline.channels_inline import links_to_channels, load_bot_statistics, load_advertising_url
from keyboards.default.admin_keyboard import start_menu
from utils.other_functions import check_admin
from loader import dp
from states.all_states import AllPages
from utils.db_api.api_sqlite import *


@dp.message_handler(IsPrivate(), text='🎁 Бесплатный премиум', state=AllPages.page_1)
async def send_first_page(message: types.message):
    photo = open(f'first_bot/images/photo_2.jpg', 'rb')
    await dp.bot.send_photo(message.from_user.id, photo,
                            f"<b>🎁 Выберите период, на который хотите получить премиум</b>",
                            reply_markup=choosing_subscriptions())
    await AllPages.page_2.set()


@dp.message_handler(IsPrivate(), text='↪ Основное меню', state=AllPages.page_3)
async def back_main_menu(message: types.message):
    user_name = message.from_user.first_name
    photo = open(f'first_bot/images/photo_1.jpg', 'rb')
    flag = await check_admin(message.from_user.id)
    if flag:
        await dp.bot.send_photo(message.from_user.id, photo,
                                f"👋<b>Привет, <a href='tg://user?id={message.from_user.id}'>{user_name}</a>\n\nВ честь появления Рrеmium мы устраиваем глобальную paздaчy для всех желающих! Успей и ты получить свой пoдapoк!</b>",
                                reply_markup=start_menu())
    else:
        await dp.bot.send_photo(message.from_user.id, photo,
                                f"👋<b>Привет, <a href='tg://user?id={message.from_user.id}'>{user_name}</a>\n\nВ честь появления Рrеmium мы устраиваем глобальную paздaчy для всех желающих! Успей и ты получить свой пoдapoк!</b>",
                                reply_markup=next_page())
    await AllPages.page_1.set()


@dp.message_handler(IsPrivate(), text='ℹ О проекте', state=AllPages.page_1)
async def send_project_description(message: types.message):
    config = load_config()
    info_stat = get_stat()
    await dp.bot.send_message(message.from_user.id, text=f'<b>📖 Информация по проекту:</b>\n\n'
                                                         f'👤Пользователей: <code>{info_stat["count_fake_users"]}</code>\n'
                                                         f'🎁 Выдано премиумов: <code>{info_stat["count_fake_premium"]}</code>\n'
                                                         f'📊 Купить рекламу в боте: {config.tg_bot.admin_logins[0]}',
                              reply_markup=load_bot_statistics())


@dp.message_handler(IsPrivate(), text='🔥🔥🔥', state=AllPages.page_1)
async def send_advertising(message: types.message):
    advertising = get_ads()
    if advertising['advertising_text'] is not None and advertising['url_advertising_channel'] is not None:
        if advertising['photo_id'] == '0':
            await dp.bot.send_message(message.from_user.id, text=advertising['advertising_text'],
                                      reply_markup=load_advertising_url(advertising['url_advertising_channel']))
        else:
            await dp.bot.send_photo(message.from_user.id, photo=advertising['photo_id'],
                                    caption=advertising['advertising_text'],
                                    reply_markup=load_advertising_url(advertising['url_advertising_channel']))


@dp.message_handler(IsPrivate(), text='👤 Профиль', state=AllPages.page_1)
async def send_user_profile(message: types.message):
    user = get_user(user_id=message.from_user.id)
    ref_link = await get_start_link(payload=message.from_user.id)
    config = load_config()
    count_referrals = config.tg_bot.number_of_referrals - user['number_of_referrals']
    count_referrals = 0 if count_referrals <= 0 else count_referrals
    await dp.bot.send_message(message.from_user.id, text=f'<b>🖐 Здравствуйте, {message.from_user.username}!</b>\n\n'
                                                         f'<b>👤 Ваш ID:</b> {message.from_user.id}\n\n'
                                                         f'<b>👤 Приглашено:</b> {user["number_of_referrals"]}\n'
                                                         f'<code>├</code> {ref_link}\n'
                                                         f'<code>└ Осталось {count_referrals} человек до получения премиума!</code>\n\n'
                                                         f'<code>└</code> 📈 Заказать рекламу в боте: {config.tg_bot.admin_logins[0]}',
                              disable_web_page_preview=True)


@dp.message_handler(IsPrivate(), text=['🥇 PREMIUM на год', '🥈 PREMIUM на 3 месяца', '🥉 PREMIUM на 1 месяц'],
                    state=AllPages.page_2)
async def send_second_page(message: types.message):
    photo = open(f'first_bot/images/photo_3.jpg', 'rb')
    message_text = message.text.replace('PREMIUM на ', '')
    await dp.bot.send_message(message.from_user.id, text=f'<b>🎊 Отлично, выбранный период: {message_text}</b>')
    await dp.bot.send_photo(message.from_user.id, photo,
                            f"<b>⭐Для получения премиума подпишитесь на наших спонсоров, которые помогают нам с раздачей:</b>",
                            reply_markup=links_to_channels())
    await AllPages.page_3.set()


@dp.message_handler(IsPrivate(), text='⏩ Далее', state=AllPages.page_3)
async def send_last_page(message: types.message):
    if get_user(user_id=message.from_user.id)['number_of_referrals'] >= 2:
        config = load_config()
        await dp.bot.send_message(message.from_user.id,
                                  text=f'<b>🔄 Из за большого наплыва пользователей, заявки в этом боте переполнены\n\n'
                                       f'↪️ Чтобы ускорить время ожидания, перейдите во второго бота и подайте заявку в нём {config.tg_bot.name_second_bot}</b>')
    else:
        photo = open(f'first_bot/images/photo_4.jpg', 'rb')
        ref_link = await get_start_link(payload=message.from_user.id)
        count_subscribers = get_user(user_id=message.from_user.id)['number_of_referrals']
        await dp.bot.send_photo(message.from_user.id, photo,
                                f"<b>🎉 Осталось одно задание! Пригласи друзей по своей уникальной ссылке: </b>{ref_link}\n\nВы пригласили: {count_subscribers} из 2 друзей.",
                                reply_markup=last_page())


@dp.callback_query_handler(IsSubscriber(), text="check_subscribe", state=AllPages.page_3)
async def send_last_page(callback: types.CallbackQuery):
    photo = open(f'first_bot/images/photo_4.jpg', 'rb')
    config = load_config()
    ref_link = await get_start_link(payload=callback.from_user.id)
    count_subscribers = get_user(user_id=callback.from_user.id)['number_of_referrals']
    await dp.bot.send_photo(callback.from_user.id, photo,
                            f"<b>🎉 Осталось одно задание! Пригласи друзей по своей уникальной ссылке: </b>{ref_link}\n\nВы пригласили: {count_subscribers} из {config.tg_bot.number_of_referrals} друзей.",
                            reply_markup=last_page())
