import sqlite3
import os

# Путь к БД
path_to_db = 'first_bot/data/database.db'


# Преобразование полученного списка в словарь
def dict_factory(cursor, row):
    save_dict = {}

    for idx, col in enumerate(cursor.description):
        save_dict[col[0]] = row[idx]

    return save_dict


# Форматирование запроса с аргументами
def update_format_args(sql, parameters: dict):
    sql = f"{sql} WHERE "

    sql += " AND ".join([
        f"{item} = ?" for item in parameters
    ])

    return sql, list(parameters.values())


# Форматирование запроса без аргументов
def update_format(sql, parameters: dict):
    if "XXX" not in sql:
        sql += " XXX "

    values = ", ".join([
        f"{item} = ?" for item in parameters
    ])
    sql = sql.replace("XXX", values)

    return sql, list(parameters.values())


# Добавление пользователя
def add_user(user_id, user_login, user_name, registration_date, referral_id, number_of_referrals):
    with sqlite3.connect(path_to_db) as db:
        db.execute("INSERT INTO users_info"
                   "(user_id, user_login, user_name, registration_date, referral_id, number_of_referrals)"
                   "VALUES (?, ?, ?, ?, ?, ?)",
                   [user_id, user_login, user_name, registration_date, referral_id, number_of_referrals])
        db.commit()


# Получение пользователя
def get_user(**kwargs):
    with sqlite3.connect(path_to_db) as db:
        db.row_factory = dict_factory
        sql = "SELECT * FROM users_info"
        sql, parameters = update_format_args(sql, kwargs)
        return db.execute(sql, parameters).fetchone()


# Удаление пользователя
def delete_user(**kwargs):
    with sqlite3.connect(path_to_db) as db:
        db.row_factory = dict_factory
        sql = "DELETE FROM users_info"
        sql, parameters = update_format_args(sql, kwargs)
        db.execute(sql, parameters)
        db.commit()


# Редактирование пользователя
def update_user(user_id, **kwargs):
    with sqlite3.connect(path_to_db) as db:
        db.row_factory = dict_factory
        sql = f"UPDATE users_info SET"
        sql, parameters = update_format(sql, kwargs)
        parameters.append(user_id)
        db.execute(sql + "WHERE user_id = ?", parameters)
        db.commit()


# Получение пользователей
def get_users(**kwargs):
    with sqlite3.connect(path_to_db) as db:
        db.row_factory = dict_factory
        sql = "SELECT * FROM users_info"
        sql, parameters = update_format_args(sql, kwargs)
        return db.execute(sql, parameters).fetchall()


# Получение всех пользователей
def get_all_users():
    with sqlite3.connect(path_to_db) as db:
        db.row_factory = dict_factory
        sql = "SELECT * FROM users_info"
        return db.execute(sql).fetchall()


# Получение статистики
def get_stat():
    with sqlite3.connect(path_to_db) as db:
        db.row_factory = dict_factory
        sql = "SELECT * FROM info_stat"
        return db.execute(sql).fetchone()


# Обновление статистики
def update_stat(**kwargs):
    with sqlite3.connect(path_to_db) as db:
        db.row_factory = dict_factory
        sql = f"UPDATE info_stat SET"
        sql, parameters = update_format(sql, kwargs)
        db.execute(sql, parameters)
        db.commit()


# Получение статистики
def get_ads():
    with sqlite3.connect(path_to_db) as db:
        db.row_factory = dict_factory
        sql = "SELECT * FROM advertising_information"
        return db.execute(sql).fetchone()


# Обновление статистики
def update_ads(**kwargs):
    with sqlite3.connect(path_to_db) as db:
        db.row_factory = dict_factory
        sql = f"UPDATE advertising_information SET"
        sql, parameters = update_format(sql, kwargs)
        db.execute(sql, parameters)
        db.commit()


# Создание всех таблиц для БД
def create_bdx():
    with sqlite3.connect(path_to_db) as db:
        # Создание БД с хранением данных пользователей
        if len(db.execute("PRAGMA table_info(users_info)").fetchall()) == 7:
            print("Загружаю базу данных. 1/3 базы загружено.")
        else:
            db.execute("CREATE TABLE users_info("
                       "increment INTEGER PRIMARY KEY AUTOINCREMENT, "
                       "user_id INTEGER, user_login TEXT, user_name TEXT, "
                       "registration_date TEXT, referral_id INTEGER, number_of_referrals INTEGER)")
            print("Создаю базу данных, 1/3.")
        # Создание БД с хранением фейковых пользователей
        if len(db.execute("PRAGMA table_info(info_stat)").fetchall()) == 2:
            print("Загружаю базу данных. 2/3 базы загружено.")
        else:
            db.execute("CREATE TABLE info_stat("
                       "count_fake_users INTEGER, count_fake_premium INTEGER)")
            sql = "INSERT INTO info_stat(" \
                  "count_fake_users, count_fake_premium) " \
                  "VALUES (?, ?)"
            db.execute(sql, (23451, 1574))
            print("Создаю базу данных, 2/3.")
        db.commit()
        # Создание БД с хранением рекламного текста и ссылки на тг канал
        if len(db.execute("PRAGMA table_info(advertising_information)").fetchall()) == 3:
            print("Загружаю базу данных. 3/3 базы загружено.")
        else:
            db.execute("CREATE TABLE advertising_information("
                       "advertising_text TEXT, url_advertising_channel TEXT, photo_id TEXT)")
            sql = "INSERT INTO advertising_information(" \
                  "advertising_text, url_advertising_channel, photo_id) " \
                  "VALUES (?, ?, ?)"
            db.execute(sql, (None, None, '0'))
            print("Создаю базу данных, 3/3.")
        db.commit()
