import sqlite3

# Путь к БД
path_to_db = "second_bot/data/database.db"


# Преобразование полученного списка в словарь
def dict_factory(cursor, row):
    save_dict = {}

    for idx, col in enumerate(cursor.description):
        save_dict[col[0]] = row[idx]

    return save_dict


# Форматирование запроса с аргументами
def update_format_args(sql, parameters: dict):
    sql = f"{sql} WHERE "

    sql += " AND ".join([
        f"{item} = ?" for item in parameters
    ])

    return sql, list(parameters.values())


# Форматирование запроса без аргументов
def update_format(sql, parameters: dict):
    if "XXX" not in sql:
        sql += " XXX "

    values = ", ".join([
        f"{item} = ?" for item in parameters
    ])
    sql = sql.replace("XXX", values)

    return sql, list(parameters.values())


# Добавление пользователя
def add_user(user_id, user_login, user_name, registration_date, list_channels):
    with sqlite3.connect(path_to_db) as db:
        db.execute("INSERT INTO users_info"
                   "(user_id, user_login, user_name, registration_date, list_channels)"
                   "VALUES (?, ?, ?, ?, ?)",
                   [user_id, user_login, user_name, registration_date, list_channels])
        db.commit()


# Получение пользователя
def get_user(**kwargs):
    with sqlite3.connect(path_to_db) as db:
        db.row_factory = dict_factory
        sql = "SELECT * FROM users_info"
        sql, parameters = update_format_args(sql, kwargs)
        return db.execute(sql, parameters).fetchone()


# Удаление пользователя
def delete_user(**kwargs):
    with sqlite3.connect(path_to_db) as db:
        db.row_factory = dict_factory
        sql = "DELETE FROM users_info"
        sql, parameters = update_format_args(sql, kwargs)
        db.execute(sql, parameters)
        db.commit()


# Редактирование пользователя
def update_user(user_id, **kwargs):
    with sqlite3.connect(path_to_db) as db:
        db.row_factory = dict_factory
        sql = f"UPDATE users_info SET"
        sql, parameters = update_format(sql, kwargs)
        parameters.append(user_id)
        db.execute(sql + "WHERE user_id = ?", parameters)
        db.commit()


# Получение пользователей
def get_users(**kwargs):
    with sqlite3.connect(path_to_db) as db:
        db.row_factory = dict_factory
        sql = "SELECT * FROM users_info"
        sql, parameters = update_format_args(sql, kwargs)
        return db.execute(sql, parameters).fetchall()


# Получение всех пользователей
def get_all_users():
    with sqlite3.connect(path_to_db) as db:
        db.row_factory = dict_factory
        sql = "SELECT * FROM users_info"
        return db.execute(sql).fetchall()


# Создание всех таблиц для БД
def create_bdx():
    with sqlite3.connect(path_to_db) as db:
        # Создание БД с хранением данных пользователей
        if len(db.execute("PRAGMA table_info(users_info)").fetchall()) == 6:
            print("Загружаю базу данных.")
        else:
            db.execute("CREATE TABLE users_info("
                       "increment INTEGER PRIMARY KEY AUTOINCREMENT, "
                       "user_id INTEGER, user_login TEXT, user_name TEXT, "
                       "registration_date TEXT, list_channels TEXT)")
            print("Создаю базу данных")
        db.commit()
